package com.iconplus.smartproc.repository;

import com.iconplus.smartproc.model.entity.Users;
import com.iconplus.smartproc.model.projection.UserRoleView;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.Optional;

@Repository
public interface UsersRepository extends JpaRepository<Users, Long> {

    @Query("SELECT u.id as id, " +
            "u.nama as nama, " +
            "u.jabatan as jabatan, " +
            "r.role as role, " +
            "u.email as email, " +
            "u.isActive as isActive, " +
            "u.isDelete as isDelete " +
            "FROM Users u " +
            "JOIN Roles r ON r.id = u.roleId " +
            "Where u.isDelete = false " +
            "AND (:search='' " +
            "or (UPPER(u.nama) like :search " +
            "   OR UPPER(u.jabatan) like :search " +
            "   OR UPPER(r.role) like :search " +
            "   OR UPPER(u.email) like :search) " +
            ") " +
            "ORDER by u.id")
    Page<UserRoleView> getAllUsersAndDeletedFase(String search, Pageable pageable);

    Optional<Users> findByEmailAndIsDeleteFalse(String email);
    
}
